@@ -1,4 +1,4 @@
-/* gentree version 1.3; Brendan McKay Oct 2022 */
+/* gentree version 1.4; Brendan McKay Dec 2022 */
 /* This program is a wrapper for the program FreeTrees.c written
  * by Gang Li & Frank Ruskey.  See below for their original
  * comments. */
@@ -32,49 +32,54 @@ Counts for n=1..45:
  1: 1
  2: 1
  3: 1
- 4: 1
- 5: 2
- 6: 3
- 7: 6
- 8: 11
- 9: 23
-10: 47
-11: 106
-12: 235
-13: 551
-14: 1301
-15: 3159
-16: 7741
-17: 19320
-18: 48629
-19: 123867
-20: 317955
-21: 823065
-22: 2144505
-23: 5623756
-24: 14828074
-25: 39299897
-26: 104636890
-27: 279793450
-28: 751065460
-29: 2023443032
-30: 5469566585
-31: 14830871802
-32: 40330829030
-33: 109972410221
-34: 300628862480
-35: 823779631721
-36: 2262366343746
-37: 6226306037178
-38: 17169677490714
-39: 47436313524262
-40: 131290543779126
-41: 363990257783343
-42: 1010748076717151
-43: 2810986483493475
-44: 7828986221515605
-45: 21835027912963086
-********************************/
+ 4: 2
+ 5: 3
+ 6: 6
+ 7: 11
+ 8: 23
+ 9: 47
+10: 106
+11: 235
+12: 551
+13: 1301
+14: 3159
+15: 7741
+16: 19320
+17: 48629
+18: 123867
+19: 317955
+20: 823065
+21: 2144505
+22: 5623756
+23: 14828074
+24: 39299897
+25: 104636890
+26: 279793450
+27: 751065460
+28: 2023443032
+29: 5469566585
+30: 14830871802
+31: 40330829030
+32: 109972410221
+33: 300628862480
+34: 823779631721
+35: 2262366343746
+36: 6226306037178
+37: 17169677490714
+38: 47436313524262
+39: 131290543779126
+40: 363990257783343
+41: 1010748076717151
+42: 2810986483493475
+43: 7828986221515605
+44: 21835027912963086
+45: 60978390985918906
+46: 170508699155987862
+47: 477355090753926460
+48: 1337946100045842285
+49: 3754194185716399992
+50: 10545233702911509534
+*******************************/
 
 /* Comments on original program by original authors */
 /*==============================================================*/
@@ -101,7 +106,8 @@ Counts for n=1..45:
                       Check MAXOUTLEN if more than 1000 */
 #include "gtools.h"
 
-#define NAUTY_PGM  4   /* 1 = geng, 2 = genbg, 3 = gentourng, 4 = gentreeg */
+  /* 1 = geng, 2 = genbg, 3 = gentourng, 4 = gentreeg, 5 = genktreeg */
+#define NAUTY_PGM  4 
 
 /*****************************************************************
 
@@ -489,7 +495,6 @@ main(int argc, char *argv[])
     long Z1,Z2,Z1arg,Z2arg;
     char *outfilename,sw;
     int i,j,argnum;
-    int splitlevinc;
     int minnv,maxnv;
     double t1,t2;
     int maxdegarg;
@@ -502,8 +507,6 @@ main(int argc, char *argv[])
     qswitch = Dswitch = gotmr = gotf = irred = FALSE;
     outfilename = NULL;
 
-    splitlevinc = 0;
-    
     argnum = 0;
     for (j = 1; !badargs && j < argc; ++j)
     {
@@ -623,11 +626,7 @@ PLUGIN_INIT
         outfile = stdout;
     }
     else if ((outfile = fopen(outfilename,"w")) == NULL)
-    {
-        fprintf(stderr,
-              ">E gentree: can't open %s for writing\n",outfilename);
-        gt_abort(NULL);
-    }
+        gt_abort_1(">E gentree: can't open %s for writing\n",outfilename);
 
     if (!qswitch)
     {
@@ -676,7 +675,7 @@ PLUGIN_INIT
         }
         else if (nv == 2)
         {
-            if (res == 0 && maxdeg >= 1 && mindiam <= 1 && maxdiam >= 2)
+            if (res == 0 && maxdeg >= 1 && mindiam <= 1 && maxdiam >= 1)
             {
                 par[1] = 0;
                 par[2] = 1;
