@@ -1,6 +1,6 @@
 /* testg.c : Find properties of graphs.  This is the source file for
    both pickg (select by property) and countg (count by property).
-   Version 2.2 of June 2022. */
+   Version 2.5 of Aug 2023. */
 /* TODO - write a header if input has one;
           Fix clunky practice of storing a pointer in a long long. */
 
@@ -52,16 +52,22 @@
      -g#  girth (0=acyclic)            -Y#  total number of cycles\n\
      -h#  maximum independent set      -k#  maximum clique\n\
      -T#  number of triangles          -K#  number of maximal cliques\n\
-     -TT# number independent sets of size 3\n\
+     -TT# number independent 3-sets    -P#  number of 5-cycles\n\
      -B#  smallest possible first side of a bipartition (0 if nonbipartite)\n\
      -H#  number of induced cycles     -W#  number of 4-cycles\n\
      -E   Eulerian (all degrees are even, connectivity not required)\n\
      -a#  group size  -o# orbits  -F# fixed points  -t vertex-transitive\n\
-     -c#  connectivity (only implemented for 0,1,2).\n\
-     -i#  min common nbrs of adjacent vertices;     -I# maximum\n\
-     -j#  min common nbrs of non-adjacent vertices; -J# maximum\n\
+     -c#  connectivity (2 means 2 or more).\n\
+     -kk# #-tree, otherwise 0. The complete graph K_n is tabulated as\n\
+           an n-tree, but matches either n-1 or n,\n\
+     -i#  min common nbrs of adjacent vertices;     -ii# maximum\n\
+     -j#  min common nbrs of non-adjacent vertices; -jj# maximum\n\
      -x#  number of sources            -xx#  number of sinks\n\
      -WW# number of diamonds\n\
+     -N#  chromatic number (limited to WORDSIZE colours)\n\
+     -NN# chromatic index (limited to max degree WORDSIZE-1)\n\
+     -AA# class (chromatic index - maximum degree + 1)\n\
+     -G#  connectivity                 -GG# edge connectivity\n\
 \n\
   Sort keys:\n\
      Counts are made for all graphs passing the constraints.  Counts\n\
@@ -76,15 +82,23 @@
   line for each value. For example --e:zZ will give the ranges of\n\
   radius and diameter that occur for each number of edges.\n\
   The output format matches the input, except that sparse6 is used\n\
-  to output an incremental graph whose predecessor is not output.\n"
+  to output an incremental graph whose predecessor is not output.\n\
+\n\
+  Some sort keys have boolean variants with parameters:\n\
+   --N#  #-colourable (i.e. chromatic number <= #)\n\
+   --A#  #-edge colourable\n\
+   --G#  #-connected (i.e. connectivity >= #)\n\
+   --GG# #-edge connected\n"
 
 #include "gtools.h"
 #include "gutils.h"
 #include "nautinv.h"
 #include "nautycliquer.h"
+#include "nauchromatic.h"
+#include "nauconnect.h"
 
 /*
-Available letters: wy AGNOP
+Available letters: wy GIJO
 
 How to add a new property:
 
@@ -162,7 +176,7 @@ typedef long value_t;
 #endif
 
 #undef CMASK
-#define CMASK(i) (((value_t)1) << (i))
+#define CMASK(i) (1LLU << (i))
 
 /* Constraints are indicated by a single letter or a letter doubled.
    The difference is shown by the doubleletter field. */
@@ -173,105 +187,173 @@ static struct constraint_st    /* Table of Constraints */
     boolean doubleletter;  /* Is a doubled letter, such as LL */
     boolean digraphok;   /* Can be computed for digraphs */
     boolean rangeonly;   /* Display a range */
-    int needed;     /* 1 = sortkey, 2 = constraint; 3 = both */
+    int needed;     /* 1 = sortkey, 2 = constraint; 4 = parameterized */
     boolean computed;
     boolean inverse;
-    nauty_counter prereq;  /* Must be earlier,
-                              must be <= bits in nauty_counter */
+    unsigned long long prereq;  /* Must be earlier,
+                              must be <= bits in unsigned long long */
     value_t lo,hi;
     char *id;
+    char *paramid;  /* NULL if not allowed; must have %d */
+    int param;      /* value of the parameter */
+    boolean paramval;   /* truth of the parameterised invariant */
     int valtype;
     value_t val;
 } constraint[] = {
 #define I_n 0
-   {'n',FALSE,TRUE,FALSE,0,FALSE,FALSE,0,-NOLIMIT,NOLIMIT,"n",INTTYPE,0}, /* always known */
+   {'n',FALSE,TRUE,FALSE,0,FALSE,FALSE,0,
+        -NOLIMIT,NOLIMIT,"n",NULL,0,FALSE,INTTYPE,0}, /* always known */
 #define I_e 1
-   {'e',FALSE,TRUE,FALSE,0,FALSE,FALSE,0,-NOLIMIT,NOLIMIT,"e",INTTYPE,0},
+   {'e',FALSE,TRUE,FALSE,0,FALSE,FALSE,0,
+        -NOLIMIT,NOLIMIT,"e",NULL,0,FALSE,INTTYPE,0},
 #define I_L 2
-   {'L',FALSE,TRUE,FALSE,0,FALSE,FALSE,CMASK(I_e),-NOLIMIT,NOLIMIT,"loops",INTTYPE,0},
+   {'L',FALSE,TRUE,FALSE,0,FALSE,FALSE,CMASK(I_e),
+        -NOLIMIT,NOLIMIT,"loops",NULL,0,FALSE,INTTYPE,0},
 #define I_d 3
-   {'d',FALSE,TRUE,FALSE,0,FALSE,FALSE,CMASK(I_e),-NOLIMIT,NOLIMIT,"mindeg",INTTYPE,0},
+   {'d',FALSE,TRUE,FALSE,0,FALSE,FALSE,CMASK(I_e),
+        -NOLIMIT,NOLIMIT,"mindeg",NULL,0,FALSE,INTTYPE,0},
 #define I_D 4
-   {'D',FALSE,TRUE,FALSE,0,FALSE,FALSE,CMASK(I_e),-NOLIMIT,NOLIMIT,"maxdeg",INTTYPE,0},
+   {'D',FALSE,TRUE,FALSE,0,FALSE,FALSE,CMASK(I_e),
+        -NOLIMIT,NOLIMIT,"maxdeg",NULL,0,FALSE,INTTYPE,0},
 #define I_u 5
-   {'u',FALSE,TRUE,FALSE,0,FALSE,FALSE,CMASK(I_e),-NOLIMIT,NOLIMIT,"minindeg",INTTYPE,0},
+   {'u',FALSE,TRUE,FALSE,0,FALSE,FALSE,CMASK(I_e),
+        -NOLIMIT,NOLIMIT,"minindeg",NULL,0,FALSE,INTTYPE,0},
 #define I_U 6
-   {'U',FALSE,TRUE,FALSE,0,FALSE,FALSE,CMASK(I_e),-NOLIMIT,NOLIMIT,"maxindeg",INTTYPE,0},
+   {'U',FALSE,TRUE,FALSE,0,FALSE,FALSE,CMASK(I_e),
+        -NOLIMIT,NOLIMIT,"maxindeg",NULL,0,FALSE,INTTYPE,0},
 #define I_m 7
-   {'m',FALSE,TRUE,FALSE,0,FALSE,FALSE,CMASK(I_e),-NOLIMIT,NOLIMIT,"minverts",INTTYPE,0},
+   {'m',FALSE,TRUE,FALSE,0,FALSE,FALSE,CMASK(I_e),
+        -NOLIMIT,NOLIMIT,"minverts",NULL,0,FALSE,INTTYPE,0},
 #define I_M 8
-   {'M',FALSE,TRUE,FALSE,0,FALSE,FALSE,CMASK(I_e),-NOLIMIT,NOLIMIT,"maxverts",INTTYPE,0},
+   {'M',FALSE,TRUE,FALSE,0,FALSE,FALSE,CMASK(I_e),
+        -NOLIMIT,NOLIMIT,"maxverts",NULL,0,FALSE,INTTYPE,0},
 #define I_s 9
-   {'s',FALSE,TRUE,FALSE,0,FALSE,FALSE,CMASK(I_e),-NOLIMIT,NOLIMIT,"mininverts",INTTYPE,0},
+   {'s',FALSE,TRUE,FALSE,0,FALSE,FALSE,CMASK(I_e),
+        -NOLIMIT,NOLIMIT,"mininverts",NULL,0,FALSE,INTTYPE,0},
 #define I_S 10
-   {'S',FALSE,TRUE,FALSE,0,FALSE,FALSE,CMASK(I_e),-NOLIMIT,NOLIMIT,"maxinverts",INTTYPE,0},
+   {'S',FALSE,TRUE,FALSE,0,FALSE,FALSE,CMASK(I_e),
+        -NOLIMIT,NOLIMIT,"maxinverts",NULL,0,FALSE,INTTYPE,0},
 #define I_E 11
-   {'E',FALSE,TRUE,FALSE,0,FALSE,FALSE,CMASK(I_e),-NOLIMIT,NOLIMIT,"eulerian",BOOLTYPE,0},
+   {'E',FALSE,TRUE,FALSE,0,FALSE,FALSE,CMASK(I_e),
+        -NOLIMIT,NOLIMIT,"eulerian",NULL,0,FALSE,BOOLTYPE,0},
 #define I_r 12
-   {'r',FALSE,TRUE,FALSE,0,FALSE,FALSE,CMASK(I_e),-NOLIMIT,NOLIMIT,"regular",BOOLTYPE,0},
+   {'r',FALSE,TRUE,FALSE,0,FALSE,FALSE,CMASK(I_e),
+        -NOLIMIT,NOLIMIT,"regular",NULL,0,FALSE,BOOLTYPE,0},
 #define I_B 13
-   {'B',FALSE,FALSE,FALSE,0,FALSE,FALSE,CMASK(I_e),-NOLIMIT,NOLIMIT,"bipside",INTTYPE,0},
+   {'B',FALSE,FALSE,FALSE,0,FALSE,FALSE,CMASK(I_e),
+        -NOLIMIT,NOLIMIT,"bipside",NULL,0,FALSE,INTTYPE,0},
 #define I_z 14
-   {'z',FALSE,TRUE,FALSE,0,FALSE,FALSE,0,-NOLIMIT,NOLIMIT,"radius",INTTYPE,0},
+   {'z',FALSE,TRUE,FALSE,0,FALSE,FALSE,0,
+        -NOLIMIT,NOLIMIT,"radius",NULL,0,FALSE,INTTYPE,0},
 #define I_Z 15
-   {'Z',FALSE,TRUE,FALSE,0,FALSE,FALSE,CMASK(I_z),-NOLIMIT,NOLIMIT,"diameter",INTTYPE,0},
+   {'Z',FALSE,TRUE,FALSE,0,FALSE,FALSE,CMASK(I_z),
+        -NOLIMIT,NOLIMIT,"diameter",NULL,0,FALSE,INTTYPE,0},
 #define I_a 16
-   {'a',FALSE,TRUE,FALSE,0,FALSE,FALSE,0,-NOLIMIT,NOLIMIT,"groupsize",GROUPSIZE,0},
+   {'a',FALSE,TRUE,FALSE,0,FALSE,FALSE,0,
+        -NOLIMIT,NOLIMIT,"groupsize",NULL,0,FALSE,GROUPSIZE,0},
 #define I_o 17
-   {'o',FALSE,TRUE,FALSE,0,FALSE,FALSE,CMASK(I_a),-NOLIMIT,NOLIMIT,"orbits",INTTYPE,0},
+   {'o',FALSE,TRUE,FALSE,0,FALSE,FALSE,CMASK(I_a),
+        -NOLIMIT,NOLIMIT,"orbits",NULL,0,FALSE,INTTYPE,0},
 #define I_t 18
-   {'t',FALSE,TRUE,FALSE,0,FALSE,FALSE,CMASK(I_o),-NOLIMIT,NOLIMIT,"transitive",BOOLTYPE,0},
+   {'t',FALSE,TRUE,FALSE,0,FALSE,FALSE,CMASK(I_o),
+        -NOLIMIT,NOLIMIT,"transitive",NULL,0,FALSE,BOOLTYPE,0},
 #define I_c 19
-   {'c',FALSE,FALSE,FALSE,0,FALSE,FALSE,0,-NOLIMIT,NOLIMIT,"connectivity",INTTYPE,0},
+   {'c',FALSE,FALSE,FALSE,0,FALSE,FALSE,0,
+        -NOLIMIT,NOLIMIT,"connectivity",NULL,0,FALSE,INTTYPE,0},
 #define I_F 20
-   {'F',FALSE,TRUE,FALSE,0,FALSE,FALSE,CMASK(I_a),-NOLIMIT,NOLIMIT,"fixedpts",INTTYPE,0},
+   {'F',FALSE,TRUE,FALSE,0,FALSE,FALSE,CMASK(I_a),
+        -NOLIMIT,NOLIMIT,"fixedpts",NULL,0,FALSE,INTTYPE,0},
 #define I_g 21
-   {'g',FALSE,FALSE,FALSE,0,FALSE,FALSE,0,-NOLIMIT,NOLIMIT,"girth",INTTYPE,0},
+   {'g',FALSE,FALSE,FALSE,0,FALSE,FALSE,0,
+        -NOLIMIT,NOLIMIT,"girth",NULL,0,FALSE,INTTYPE,0},
 #define I_Y 22
-   {'Y',FALSE,FALSE,FALSE,0,FALSE,FALSE,0,-NOLIMIT,NOLIMIT,"cycles",INTTYPE,0},
+   {'Y',FALSE,FALSE,FALSE,0,FALSE,FALSE,0,
+        -NOLIMIT,NOLIMIT,"cycles",NULL,0,FALSE,INTTYPE,0},
 #define I_i 23
-   {'i',FALSE,FALSE,FALSE,0,FALSE,FALSE,0,-NOLIMIT,NOLIMIT,"minadjcn",INTTYPE,0},
-#define I_I 24
-   {'I',FALSE,FALSE,FALSE,0,FALSE,FALSE,CMASK(I_i),-NOLIMIT,NOLIMIT,"maxadjcn",INTTYPE,0},
+   {'i',FALSE,FALSE,FALSE,0,FALSE,FALSE,0,
+        -NOLIMIT,NOLIMIT,"minadjcn",NULL,0,FALSE,INTTYPE,0},
+#define I_ii 24
+   {'i',TRUE,FALSE,FALSE,0,FALSE,FALSE,CMASK(I_i),
+        -NOLIMIT,NOLIMIT,"maxadjcn",NULL,0,FALSE,INTTYPE,0},
 #define I_j 25
-   {'j',FALSE,FALSE,FALSE,0,FALSE,FALSE,CMASK(I_i),-NOLIMIT,NOLIMIT,"minnoncn",INTTYPE,0},
-#define I_J 26
-   {'J',FALSE,FALSE,FALSE,0,FALSE,FALSE,CMASK(I_i),-NOLIMIT,NOLIMIT,"maxnoncn",INTTYPE,0},
+   {'j',FALSE,FALSE,FALSE,0,FALSE,FALSE,CMASK(I_i),
+        -NOLIMIT,NOLIMIT,"minnoncn",NULL,0,FALSE,INTTYPE,0},
+#define I_jj 26
+   {'j',TRUE,FALSE,FALSE,0,FALSE,FALSE,CMASK(I_i),
+        -NOLIMIT,NOLIMIT,"maxnoncn",NULL,0,FALSE,INTTYPE,0},
 #define I_T 27
-   {'T',FALSE,TRUE,FALSE,0,FALSE,FALSE,0,-NOLIMIT,NOLIMIT,"triang",INTTYPE,0},
+   {'T',FALSE,TRUE,FALSE,0,FALSE,FALSE,0,
+        -NOLIMIT,NOLIMIT,"triang",NULL,0,FALSE,INTTYPE,0},
 #define I_K 28
-   {'K',FALSE,FALSE,FALSE,0,FALSE,FALSE,0,-NOLIMIT,NOLIMIT,"maxlcliq",INTTYPE,0},
+   {'K',FALSE,FALSE,FALSE,0,FALSE,FALSE,0,
+        -NOLIMIT,NOLIMIT,"maxlcliq",NULL,0,FALSE,INTTYPE,0},
 #define I_H 29
-   {'H',FALSE,FALSE,FALSE,0,FALSE,FALSE,0,-NOLIMIT,NOLIMIT,"induced cycles",INTTYPE,0},
+   {'H',FALSE,FALSE,FALSE,0,FALSE,FALSE,0,
+        -NOLIMIT,NOLIMIT,"induced cycles",NULL,0,FALSE,INTTYPE,0},
 #define I_b 30
-   {'b',FALSE,FALSE,FALSE,0,FALSE,FALSE,0,-NOLIMIT,NOLIMIT,"bipartite",BOOLTYPE,0},
+   {'b',FALSE,FALSE,FALSE,0,FALSE,FALSE,0,
+        -NOLIMIT,NOLIMIT,"bipartite",NULL,0,FALSE,BOOLTYPE,0},
 #define I_C 31
-   {'C',FALSE,TRUE,FALSE,0,FALSE,FALSE,0,-NOLIMIT,NOLIMIT,"strong",BOOLTYPE,0},
+   {'C',FALSE,TRUE,FALSE,0,FALSE,FALSE,0,
+        -NOLIMIT,NOLIMIT,"strong",NULL,0,FALSE,BOOLTYPE,0},
 #define I_h 32
-   {'h',FALSE,FALSE,FALSE,0,FALSE,FALSE,CMASK(I_e),-NOLIMIT,NOLIMIT,"maxindset",INTTYPE,0},
+   {'h',FALSE,FALSE,FALSE,0,FALSE,FALSE,CMASK(I_e),
+        -NOLIMIT,NOLIMIT,"maxindset",NULL,0,FALSE,INTTYPE,0},
 #define I_k 33
-   {'k',FALSE,FALSE,FALSE,0,FALSE,FALSE,CMASK(I_e),-NOLIMIT,NOLIMIT,"maxclique",INTTYPE,0},
+   {'k',FALSE,FALSE,FALSE,0,FALSE,FALSE,CMASK(I_e),
+        -NOLIMIT,NOLIMIT,"maxclique",NULL,0,FALSE,INTTYPE,0},
 #define I_LL 34
-   {'L',TRUE,TRUE,FALSE,0,FALSE,FALSE,CMASK(I_e),-NOLIMIT,NOLIMIT,"digons",INTTYPE,0},
+   {'L',TRUE,TRUE,FALSE,0,FALSE,FALSE,CMASK(I_e),
+        -NOLIMIT,NOLIMIT,"digons",NULL,0,FALSE,INTTYPE,0},
 #define I_cc 35
-   {'c',TRUE,FALSE,FALSE,0,FALSE,FALSE,0,-NOLIMIT,NOLIMIT,"components",INTTYPE,0},
+   {'c',TRUE,FALSE,FALSE,0,FALSE,FALSE,0,
+        -NOLIMIT,NOLIMIT,"components",NULL,0,FALSE,INTTYPE,0},
 #define I_ee 36
-   {'e',TRUE,TRUE,FALSE,0,FALSE,FALSE,CMASK(I_e),-NOLIMIT,NOLIMIT,"nonedges",INTTYPE,0},
+   {'e',TRUE,TRUE,FALSE,0,FALSE,FALSE,CMASK(I_e),
+        -NOLIMIT,NOLIMIT,"nonedges",NULL,0,FALSE,INTTYPE,0},
 #define I_TT 37
-   {'T',TRUE,FALSE,FALSE,0,FALSE,FALSE,0,-NOLIMIT,NOLIMIT,"ind3sets",INTTYPE,0},
+   {'T',TRUE,FALSE,FALSE,0,FALSE,FALSE,0,
+        -NOLIMIT,NOLIMIT,"ind3sets",NULL,0,FALSE,INTTYPE,0},
 #define I_x 38
-   {'x',FALSE,TRUE,FALSE,0,FALSE,FALSE,0,-NOLIMIT,NOLIMIT,"sources",INTTYPE,0},
+   {'x',FALSE,TRUE,FALSE,0,FALSE,FALSE,0,
+        -NOLIMIT,NOLIMIT,"sources",NULL,0,FALSE,INTTYPE,0},
 #define I_xx 39
-   {'x',TRUE,TRUE,FALSE,0,FALSE,FALSE,CMASK(I_x),-NOLIMIT,NOLIMIT,"sinks",INTTYPE,0},
+   {'x',TRUE,TRUE,FALSE,0,FALSE,FALSE,CMASK(I_x),
+        -NOLIMIT,NOLIMIT,"sinks",NULL,0,FALSE,INTTYPE,0},
 #define I_W 40
-   {'W',FALSE,FALSE,FALSE,0,FALSE,FALSE,0,-NOLIMIT,NOLIMIT,"squares",INTTYPE,0},
+   {'W',FALSE,FALSE,FALSE,0,FALSE,FALSE,0,
+        -NOLIMIT,NOLIMIT,"squares",NULL,0,FALSE,INTTYPE,0},
 #define I_WW 41
-   {'W',TRUE,FALSE,FALSE,0,FALSE,FALSE,0,-NOLIMIT,NOLIMIT,"diamonds",INTTYPE,0},
-#define I_Q 42
+   {'W',TRUE,FALSE,FALSE,0,FALSE,FALSE,0,
+        -NOLIMIT,NOLIMIT,"diamonds",NULL,0,FALSE,INTTYPE,0},
+#define I_P 42
+   {'P',FALSE,FALSE,FALSE,0,FALSE,FALSE,0,
+        -NOLIMIT,NOLIMIT,"pentagons",NULL,0,FALSE,INTTYPE,0},
+#define I_kk 43
+   {'k',TRUE,FALSE,FALSE,0,FALSE,FALSE,0,
+        -NOLIMIT,NOLIMIT,"ktree",NULL,0,FALSE,INTTYPE,0},
+#define I_N 44
+   {'N',FALSE,FALSE,FALSE,0,FALSE,FALSE,0,
+        -NOLIMIT,NOLIMIT,"chrom","%d-colourable",0,FALSE,INTTYPE,0},
+#define I_NN 45
+   {'N',TRUE,FALSE,FALSE,0,FALSE,FALSE,0,
+        -NOLIMIT,NOLIMIT,"echrom","%d-edge colourable",0,FALSE,INTTYPE,0},
+#define I_A 46
+   {'A',FALSE,FALSE,FALSE,0,FALSE,FALSE,0,
+        -NOLIMIT,NOLIMIT,"class",NULL,0,FALSE,INTTYPE,0},
+#define I_G 47
+   {'G',FALSE,TRUE,FALSE,0,FALSE,FALSE,0,
+        -NOLIMIT,NOLIMIT,"connect","%d-connected",0,FALSE,INTTYPE,0},
+#define I_GG 48
+   {'G',TRUE,TRUE,FALSE,0,FALSE,FALSE,0,
+        -NOLIMIT,NOLIMIT,"econnect","%d-edge connected",0,FALSE,INTTYPE,0},
+#define I_Q 49
 #if defined(USERDEF) || defined(LUSERDEF)
-   {'Q',FALSE,QDIGRAPH,FALSE,0,FALSE,FALSE,0,-NOLIMIT,NOLIMIT,USERDEFNAME,INTTYPE,0}
+   {'Q',FALSE,QDIGRAPH,FALSE,0,FALSE,FALSE,0,
+        -NOLIMIT,NOLIMIT,USERDEFNAME,NULL,0,FALSE,INTTYPE,0}
 #else
-   {' ',FALSE,FALSE,FALSE,0,FALSE,FALSE,0,-NOLIMIT,NOLIMIT,USERDEFNAME,INTTYPE,0}
+   {' ',FALSE,FALSE,FALSE,0,FALSE,FALSE,0,
+        -NOLIMIT,NOLIMIT,USERDEFNAME,NULL,0,FALSE,INTTYPE,0}
 #endif
 };
 
@@ -280,8 +362,9 @@ static struct constraint_st    /* Table of Constraints */
 #define ISDOUBLED(i) (constraint[i].doubleletter)
 #define ISNEEDED(i) (constraint[i].needed > 0)
 #define NEEDED(i) (constraint[i].needed)
-#define ISKEY(i) ((constraint[i].needed & 1) != 0)
-#define ISCONSTRAINT(i) (constraint[i].needed > 1)
+#define ISKEY(i) ((constraint[i].needed & 1))
+#define ISCONSTRAINT(i) ((constraint[i].needed & 2))
+#define ISPARAMETERIZED(i) ((constraint[i].needed & 4))
 #define ISRANGEONLY(i) (constraint[i].rangeonly)
 #define INVERSE(i) (constraint[i].inverse)
 #define COMPUTED(i) (constraint[i].computed)
@@ -291,6 +374,9 @@ static struct constraint_st    /* Table of Constraints */
 #define VAL(i) (constraint[i].val)
 #define VALTYPE(i) (constraint[i].valtype)
 #define ID(i) (constraint[i].id)
+#define PARAMID(i) (constraint[i].paramid)
+#define PARAM(i) (constraint[i].param)
+#define PARAMVAL(i) (constraint[i].paramval)
 #define ISDIGOK(i) (constraint[i].digraphok)
 
 #define INBOUNDS0(i) ((LO(i) == -NOLIMIT || VAL(i) >= LO(i)) \
@@ -312,7 +398,11 @@ static boolean rangemarkerseen = FALSE;  /* --: seen */
 typedef struct splay_st
 {
     struct splay_st *left,*right,*parent;
+#if FLEX_ARRAY_OK
+    value_t data[];
+#else
     value_t data[1];
+#endif
 } splay_node;
 
 typedef struct range_st
@@ -323,7 +413,7 @@ typedef struct range_st
 typedef struct node_st     /* variant for count tree */
 {
     struct splay_st *left,*right,*parent;
-    nauty_counter count;
+    unsigned long long count;
     range val[MAXKEYS];       /* Need this big? */
 } count_node;
 
@@ -331,7 +421,11 @@ typedef struct value_st    /* variant for value tree */
 {
     struct splay_st *left,*right,*parent;
     size_t size;
+#if FLEX_ARRAY_OK
+    value_t data[];
+#else
     value_t data[1];
+#endif
 } value_node;
 
 #define SPLAYNODE splay_node
@@ -350,7 +444,7 @@ typedef struct value_st    /* variant for value tree */
 #define NOT_PRESENT(p) {memcpy((void*)p,(void*)new_val,SPLAYNODESIZE); \
                 if (!isvalue) TOCOUNT(p)->count = 1;}
 
-static void printkeyvals(FILE*,nauty_counter,range*);
+static void printkeyvals(FILE*,unsigned long long,range*);
 static void possibleblankline(FILE*,range*);
 static int compare_count_node(count_node*,count_node*);
 static int add_count(count_node*,count_node*);
@@ -498,7 +592,8 @@ add_one(void)
     count_node new_val;
 
     for (i = 0; i < numkeys; ++i)
-        new_val.val[i].lo = new_val.val[i].hi = VAL(key[i]);
+        new_val.val[i].lo = new_val.val[i].hi
+            = (ISPARAMETERIZED(key[i]) ? PARAMVAL(key[i]) : VAL(key[i]));
 
     splay_insert(&count_root,FALSE,TOSPLAY(&new_val),
         offsetof(count_node,val) + numkeys*sizeof(range));
@@ -518,7 +613,12 @@ printthesevals(FILE *f)
             ki = key[i];
             if (i > 0) fprintf(f," ");
 
-            if (VALTYPE(ki) == BOOLTYPE)
+            if (ISPARAMETERIZED(ki))
+            {
+                if (!PARAMVAL(ki)) fprintf(f,"0");
+                else               fprintf(f,"1");
+            }
+            else if (VALTYPE(ki) == BOOLTYPE)
             {
                 if (!VAL(ki)) fprintf(f,"0");
                 else          fprintf(f,"1");
@@ -536,7 +636,12 @@ printthesevals(FILE *f)
             ki = key[i];
             if (i > 0) fprintf(f,"; ");
 
-            if (VALTYPE(ki) == BOOLTYPE)
+            if (ISPARAMETERIZED(ki))
+            {
+                if (!PARAMVAL(ki)) fprintf(f,"not %s",PARAMID(ki));
+                else               fprintf(f,"%s",PARAMID(ki));
+            }
+            else if (VALTYPE(ki) == BOOLTYPE)
             {
                 if (!VAL(ki)) fprintf(f,"not %s",ID(ki));
                 else          fprintf(f,"%s",ID(ki));
@@ -568,7 +673,7 @@ possibleblankline(FILE *f, range *val)
 /**********************************************************************/
 
 static void
-printkeyvals(FILE *f, nauty_counter count, range *val)
+printkeyvals(FILE *f, unsigned long long count, range *val)
 {
     int i,ki;
     group_node *sza,*szb;
@@ -580,7 +685,7 @@ printkeyvals(FILE *f, nauty_counter count, range *val)
             ki = key[i];
             if (i > 0) fprintf(f," ");
 
-            if (VALTYPE(ki) == BOOLTYPE)
+            if (VALTYPE(ki) == BOOLTYPE || ISPARAMETERIZED(ki))
             {
                 if (!val[i].lo) fprintf(f,"0");
                 else            fprintf(f,"1");
@@ -612,7 +717,12 @@ printkeyvals(FILE *f, nauty_counter count, range *val)
             ki = key[i];
             if (i > 0) fprintf(f,"; ");
 
-            if (VALTYPE(ki) == BOOLTYPE)
+            if (ISPARAMETERIZED(ki))
+            {
+                if (!val[i].lo) fprintf(f,"not %s",PARAMID(ki));
+                else            fprintf(f,"%s",PARAMID(ki));
+            }
+            else if (VALTYPE(ki) == BOOLTYPE)
             {
                 if (!val[i].lo) fprintf(f,"not %s",ID(ki));
                 else            fprintf(f,"%s",ID(ki));
@@ -744,7 +854,7 @@ static void
 compute(graph *g, int m, int n, int code, boolean digraph)
 /* Compute property i assuming the prerequisites are known. */
 {
-    int mind,maxd,mincount,maxcount;
+    int mind,maxd,maxdeg,mincount,maxcount;
     int minind,maxind,minincount,maxincount;
     int rad,diam,loops;
     unsigned long ned;
@@ -752,7 +862,7 @@ compute(graph *g, int m, int n, int code, boolean digraph)
     group_node sz;
     int norbs,fixedpts;
     int minadj,maxadj,minnon,maxnon;
-    int sources,sinks;
+    int sources,sinks,lowneeded,highneeded;
 
     switch (code)
     {
@@ -831,6 +941,11 @@ compute(graph *g, int m, int n, int code, boolean digraph)
             COMPUTED(I_z) = COMPUTED(I_Z) = TRUE;
             break;
 
+        case I_kk:
+            VAL(I_kk) = ktreeness(g,m,n);
+            COMPUTED(I_kk) = TRUE;
+            break;
+
         case I_a:
             if (!COMPUTED(I_L))
             {
@@ -893,6 +1008,11 @@ compute(graph *g, int m, int n, int code, boolean digraph)
             COMPUTED(I_WW) = TRUE;
             break;
 
+        case I_P:
+            VAL(I_P) = numpentagons(g,m,n);
+            COMPUTED(I_P) = TRUE;
+            break;
+
         case I_TT:
             VAL(I_TT) = numind3sets(g,m,n);
             COMPUTED(I_TT) = TRUE;
@@ -903,19 +1023,140 @@ compute(graph *g, int m, int n, int code, boolean digraph)
             sources_sinks(g,m,n,&sources,&sinks);
             VAL(I_x) = sources;
             VAL(I_xx) = sinks;
+            COMPUTED(I_x) = COMPUTED(I_xx) = TRUE;
+            break;
+
+        case I_N:
+            lowneeded = (LO(I_N) < 1 ? 1 : LO(I_N));
+            highneeded = (HI(I_N) > n ? n : HI(I_N));
+
+            if (!ISKEY(I_N) && lowneeded == 1) lowneeded = highneeded;
+            
+            if (ISPARAMETERIZED(I_N))
+            {
+                if (PARAM(I_N) < lowneeded) lowneeded = PARAM(I_N);
+                if (highneeded == n) highneeded = PARAM(I_N);
+            }
+
+            VAL(I_N) = chromaticnumber(g,m,n,lowneeded,highneeded);
+            if (ISPARAMETERIZED(I_N))
+                PARAMVAL(I_N) = (VAL(I_N) <= PARAM(I_N));
+            COMPUTED(I_N) = TRUE;
+            break;
+
+        case I_NN:
+        case I_A:
+            VAL(I_NN) = chromaticindex(g,m,n,&maxdeg);
+            VAL(I_A) = VAL(I_NN) - maxdeg + 1;
+            if (ISPARAMETERIZED(I_NN))
+                PARAMVAL(I_NN) = (VAL(I_NN) <= PARAM(I_NN));
+            COMPUTED(I_NN) = TRUE;
+            COMPUTED(I_A) = TRUE;
+            break;
+
+        case I_G:
+            lowneeded = (LO(I_G) < 0 ? 0 : LO(I_G));
+            highneeded = (HI(I_G) > n-1 ? n-1 : HI(I_G));
+
+            if (!ISKEY(I_G))
+            {
+                if (highneeded == n-1)
+                   VAL(I_G) = (isthisconnected(g,m,n,lowneeded,digraph)
+                                ? lowneeded : lowneeded - 1);
+                else
+                   VAL(I_G) = connectivity(g,m,n,digraph);
+            }
+            else if (ISPARAMETERIZED(I_G))
+            {
+                if (PARAM(I_G) <= lowneeded)
+                {
+                    if (highneeded == n-1)
+                        VAL(I_G) = (isthisconnected(g,m,n,lowneeded,digraph)
+                                ? lowneeded : lowneeded - 1);
+                    else
+                        VAL(I_G) = connectivity(g,m,n,digraph);
+                }
+                else if (PARAM(I_G) <= highneeded)
+                {
+                    if (lowneeded == 0 && highneeded == n-1)
+                        VAL(I_G) = (isthisconnected(g,m,n,PARAM(I_G),digraph)
+                                ? PARAM(I_G) : PARAM(I_G) - 1);
+                    else
+                        VAL(I_G) = connectivity(g,m,n,digraph);
+                }
+                else
+                {
+                    if (highneeded == n-1)
+                        VAL(I_G) = (isthisconnected(g,m,n,lowneeded,digraph)
+                                ? lowneeded : lowneeded - 1);
+                    else
+                        VAL(I_G) = connectivity(g,m,n,digraph);
+                }
+                PARAMVAL(I_G) = (VAL(I_G) >= PARAM(I_G));
+            }
+            else /* --G */
+                VAL(I_G) = connectivity(g,m,n,digraph);
+
+            COMPUTED(I_G) = TRUE;
+            break;
+
+        case I_GG:
+            lowneeded = (LO(I_GG) < 0 ? 0 : LO(I_GG));
+            highneeded = (HI(I_GG) > n-1 ? n-1 : HI(I_GG));
+
+            if (!ISKEY(I_GG))
+            {
+                if (highneeded == n-1)
+                   VAL(I_GG) = (isthisedgeconnected(g,m,n,lowneeded)
+                                ? lowneeded : lowneeded - 1);
+                else
+                   VAL(I_GG) = edgeconnectivity(g,m,n);
+            }
+            else if (ISPARAMETERIZED(I_GG))
+            {
+                if (PARAM(I_GG) <= lowneeded)
+                {
+                    if (highneeded == n-1)
+                        VAL(I_GG) = (isthisedgeconnected(g,m,n,lowneeded)
+                                ? lowneeded : lowneeded - 1);
+                    else
+                        VAL(I_GG) = edgeconnectivity(g,m,n);
+                }
+                else if (PARAM(I_GG) <= highneeded)
+                {
+                    if (lowneeded == 0 && highneeded == n-1)
+                        VAL(I_GG) = (isthisedgeconnected(g,m,n,PARAM(I_GG))
+                                ? PARAM(I_GG) : PARAM(I_GG) - 1);
+                    else
+                        VAL(I_GG) = edgeconnectivity(g,m,n);
+                }
+                else
+                {
+                    if (highneeded == n-1)
+                        VAL(I_GG) = (isthisedgeconnected(g,m,n,lowneeded)
+                                ? lowneeded : lowneeded - 1);
+                    else
+                        VAL(I_GG) = edgeconnectivity(g,m,n);
+                }
+                PARAMVAL(I_GG) = (VAL(I_GG) >= PARAM(I_GG));
+            }
+            else /* --G */
+                VAL(I_GG) = edgeconnectivity(g,m,n);
+
+            COMPUTED(I_GG) = TRUE;
             break;
 
         case I_i:
-        case I_I:
+        case I_ii:
         case I_j:
-        case I_J:
+        case I_jj:
             commonnbrs(g,&minadj,&maxadj,&minnon,&maxnon,m,n);
             VAL(I_i) = minadj;
-            VAL(I_I) = maxadj;
+            VAL(I_ii) = maxadj;
             VAL(I_j) = minnon;
-            VAL(I_J) = maxnon;
-            COMPUTED(I_i) = COMPUTED(I_I) = TRUE;
-            COMPUTED(I_j) = COMPUTED(I_J) = TRUE;
+            VAL(I_jj) = maxnon;
+            COMPUTED(I_i) = COMPUTED(I_ii) = TRUE;
+            COMPUTED(I_j) = COMPUTED(I_jj) = TRUE;
             break;
 
         case I_h:
@@ -998,6 +1239,7 @@ selected(graph *g, int m, int n, boolean digraph)
 /* See if g is selected by the constraints */
 {
     int i;
+    boolean inbounds;
 
     VAL(I_n) = n;
     COMPUTED(I_n) = TRUE;
@@ -1009,7 +1251,13 @@ selected(graph *g, int m, int n, boolean digraph)
 
         if (ISCONSTRAINT(i))
         {
-            if (INBOUNDS(i))
+            inbounds = INBOUNDS(i);
+            if (i == I_kk && constraint[i].val == n
+                && ((LO(i) == -NOLIMIT || n-1 >= LO(i))
+                && (HI(i) == NOLIMIT || n-1 <= HI(i))))
+                    inbounds = TRUE;
+
+            if (inbounds)
             {
                 if (INVERSE(i)) return FALSE;
             }
@@ -1029,11 +1277,13 @@ static void
 decodekeys(char *s)
 /* Extract key symbols from -- string */
 {
-    int i,j,k;
+    int j,k,pval;
+    char *si,*pname;
+    boolean doubled;
 
-    for (i = 0; s[i] != '\0'; ++i)
+    for (si = s; *si != '\0'; ++si)
     {
-        if (s[i] == ':')
+        if (*si == ':')
         {
             if (rangemarkerseen)
             {
@@ -1044,31 +1294,43 @@ decodekeys(char *s)
             continue;
         }
 
-        if (s[i] == ',') continue;
+        if (*si == ',') continue;
+
+        if (*(si+1) == *si)
+        {
+            doubled = TRUE;
+            ++si;
+        }
+        else
+            doubled = FALSE;
 
         for (j = 0; j < NUMCONSTRAINTS; ++j)
-            if (s[i] == SYMBOL(j) &&
-                     (!ISDOUBLED(j) && s[i+1] != SYMBOL(j)
-                   || (ISDOUBLED(j) && s[i+1] == SYMBOL(j))))
+            if (*si == SYMBOL(j) &&
+                   ((!ISDOUBLED(j) && !doubled) || (ISDOUBLED(j) && doubled)))
                 break;
 
         if (j == NUMCONSTRAINTS)
         {
-            fprintf(stderr,">E unknown sort key %c\n",s[i]);
+            fprintf(stderr,">E unknown sort key ");
+            if (doubled) fprintf(stderr,"%c%c\n",*si,*si);
+            else         fprintf(stderr,"%c\n",*si);
             exit(1);
         }
-        if (ISDOUBLED(j)) ++i;
 
         if (rangemarkerseen && VALTYPE(j) == BOOLTYPE)
         {
             fprintf(stderr,
-                    ">W ignoring unsplit boolean property %c\n",s[i]);
+                    ">W ignoring unsplit boolean property %c\n",*si);
             continue;
         }
 
         for (k = 0; k < numkeys; ++k) if (key[k] == j) break;
 
-        if (k == numkeys)
+        if (k < numkeys)
+        {
+            fprintf(stderr,">W repeated sort key ignored\n");
+        }
+        else
         {
             if (numkeys == MAXKEYS)
             {
@@ -1078,6 +1340,31 @@ decodekeys(char *s)
             }
             key[numkeys++] = j;
             NEEDED(j) |= 1;
+            if (*(si+1) >= '0' && *(si+1) <= '9')
+            {
+                if (PARAMID(j) == NULL)
+                {
+                    if (doubled) fprintf(stderr,
+                                ">E --%c%c can't be parameterized\n",*si,*si);
+                    else         fprintf(stderr,
+                                ">E --%c can't be parameterized\n",*si);
+                    exit(1);
+                }
+                if (rangemarkerseen)
+                    fprintf(stderr,">W ignoring unsplit parameterized property\n");
+                else
+                {
+                    ++si;
+                    arg_int(&si,&pval,"testg --parameter");
+                    --si;
+                    PARAM(j) = pval;
+                    pname = malloc(strlen(PARAMID(j)+20));
+                    sprintf(pname,PARAMID(j),pval);
+                    PARAMID(j) = pname;
+                    NEEDED(j) |= 4;
+                }
+            }
+
             if (!rangemarkerseen) ++numsplitkeys;
         }
     }
@@ -1112,13 +1399,13 @@ main(int argc, char *argv[])
     int m,n,codetype,outcode;
     char *infilename,*outfilename;
     FILE *infile,*outfile,*countfile;
-    nauty_counter nin,nout;
+    unsigned long long nin,nout;
     int argnum,i,j,nprev,mprev,digbad;
     char *arg,sw,*baseptr,*bp;
     boolean badargs,lastwritten,digraph;
     long pval1,pval2,maxin;
     boolean fswitch,pswitch,Vswitch,vswitch,Xswitch,qswitch;
-    nauty_counter cmask;
+    unsigned long long cmask;
     long arglo,arghi;
     boolean havecon,neg,doflush,isselected;
     double t;
@@ -1158,11 +1445,8 @@ main(int argc, char *argv[])
 #endif
 
     if (!docount && !dofilter)
-    {
-        fprintf(stderr,
+        gt_abort_1(
          ">E %s: can\'t tell if this is a picker or a counter\n",argv[0]);
-        gt_abort(NULL);
-    }
 
     argnum = 0;
     badargs = FALSE;
@@ -1206,7 +1490,7 @@ main(int argc, char *argv[])
                     {
                         if (sw == SYMBOL(i) &&
                               ((ISDOUBLED(i) && *arg == sw)
-                            || !ISDOUBLED(i) && *arg != sw))
+                            || (!ISDOUBLED(i) && *arg != sw)))
                         {
                             if (ISDOUBLED(i)) ++arg;
                             NEEDED(i) |= 2;
@@ -1292,6 +1576,8 @@ main(int argc, char *argv[])
                         fprintf(stderr,":%c%c",SYMBOL(key[j]),SYMBOL(key[j]));
                     else
                         fprintf(stderr,":%c",SYMBOL(key[j]));
+                    if (ISPARAMETERIZED(key[j]))
+                        fprintf(stderr,"%d",PARAM(key[j]));
                 }
                 else
                 {
@@ -1299,6 +1585,8 @@ main(int argc, char *argv[])
                         fprintf(stderr,"%c%c",SYMBOL(key[j]),SYMBOL(key[j]));
                     else
                         fprintf(stderr,"%c",SYMBOL(key[j]));
+                    if (ISPARAMETERIZED(key[j]))
+                        fprintf(stderr,"%d",PARAM(key[j]));
                 }
         }
 
@@ -1347,10 +1635,7 @@ main(int argc, char *argv[])
         outfile = stdout;
     }
     else if ((outfile = fopen(outfilename,"w")) == NULL)
-    {
-        fprintf(stderr,"Can't open output file %s\n",outfilename);
-        gt_abort(NULL);
-    }
+        gt_abort_1(">E Can't open output file %s\n",outfilename);
 
     if (dofilter) countfile = stderr;
     else          countfile = outfile;
@@ -1372,14 +1657,17 @@ main(int argc, char *argv[])
         if (digraph && digbad >= 0)
         {
             if (ISDOUBLED(digbad))
-                fprintf(stderr,
+            {
+                gt_abort_3(
                     ">E %s: option %c%c is not implemented for digraphs\n",
                     argv[0],SYMBOL(digbad),SYMBOL(digbad));
+            }
             else
-                fprintf(stderr,
+            {
+                gt_abort_2(
                     ">E %s: option %c is not implemented for digraphs\n",
                     argv[0],SYMBOL(digbad));
-            gt_abort(NULL);
+            }
         }
 
         for (j = 0; j < NUMCONSTRAINTS; ++j) COMPUTED(j) = FALSE;
